"""
opbeat.contrib.django.celery
~~~~~~~~~~~~~~~~~~~~~~~~~~~~

:copyright: (c) 2011-2012 Opbeat

Large portions are
:copyright: (c) 2010 by the Sentry Team, see AUTHORS for more details.
:license: BSD, see LICENSE for more details.
"""

from opbeat.contrib.celery import CeleryMixin
from opbeat.contrib.django import DjangoClient

class CeleryClient(CeleryMixin, DjangoClient):
    pass